/*
*  Rubrica
*  file: search.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include <gnome.h>

#include "rubrica.h"
#include "search.h"
#include "search_int.h"
#include "error.h"

WhereSearch where;      /* where search    */
gboolean how;           /* how search      */
gchar *who;             /* who/what search */
GList *coda = NULL;
extern GtkWidget *label_result;

void rubrica_search_init()
{
  /* default */
  where = CARD;   /* search by card  */
  how = FALSE;     /* case insensitive */		 
}

void rubrica_search_where_set(WhereSearch search_where)
{
  where = (WhereSearch) search_where;
}

void rubrica_search_sensitive_set(gboolean flag)
{
  /*
    se flag = TRUE  la ricerca deve essere case sensitive      -- strcmp
    se flag = FALSE la ricerca non deve essere case sensitive  -- strcasecmp
   */  
  how = flag;
}

gboolean rubrica_match(gchar *str)
{
  gboolean flag;
  
  flag = FALSE;

  if (how)	
    {
      if(strcmp(str, who) == 0)
	flag = TRUE;
    }
  else 
    {
      if(strcasecmp(str, who) == 0)
	flag = TRUE;
    }
  return (gboolean) flag;
}

void rubrica_search_item()
{
  GList *list;
  GList *ls;
  RubricaItem *item;
  GtkWidget *entry;
  GtkCList *vista_clist;
  GtkWidget *result_clist;
  gint trovati, pos;
  gchar *str, *buffer;
  
//   entry = gnome_entry_gtk_entry(GNOME_ENTRY(user_data));

  /* pulisco la coda da una eventuale ricerca precedente */
  if (coda)
    rubrica_free_coda(coda);

  coda = NULL;
  str = NULL;
  list = rubrica_get_rubrica();

  entry = gnome_entry_gtk_entry(GNOME_ENTRY(rubrica_search_get_search_text()));
  result_clist = rubrica_search_get_search_list();
  
  vista_clist = GTK_CLIST(rubrica_vista_get_dati());

  who = gtk_entry_get_text(GTK_ENTRY(entry));
  trovati = 0;
  pos = 0;
  
  /* pulisco la finestra dei risultati della ricerca */
  gtk_clist_clear(GTK_CLIST(result_clist));

  if (who == NULL)
    error_no_string();
  else
    while (list)
      {
	gboolean find;
	
	find = FALSE;
	item = (RubricaItem*) list->data;
      
	switch(where)     
	  {
	  case CARD:
	    if (item->scheda)
	      {
	      str = g_strdup(item->scheda);
	      
	      find = rubrica_match(str);
	      }
	    break;

	  case NAME:
	    if (item->nome)
	      {
		str = g_strdup(item->nome);

		find = rubrica_match(str);
	      }
	    break;

	    case SURNAME:
	      if (item->cognome)
		{
		  str = g_strdup(item->cognome);

		  find = rubrica_match(str);
		}
	    break;

	    case STATE:
	      if (item->stato)
		{
		str = g_strdup(item->stato);

		find = rubrica_match(str);
		}
	    break;

	    case CITY:
	      if (item->citta)
		{
		str = g_strdup(item->citta);

		find = rubrica_match(str);
		}
	    break;

	    case PROF:
	      if (item->professione)
		{
		str = g_strdup(item->professione);

		find = rubrica_match(str);
		}
	    break;
	    
	  case WEB:
	    if (item->web_list)
	      {
		ls = item->web_list;
		
		while (ls)
		  {
		    str = g_strdup(ls->data);
		    
		    if ((find = rubrica_match(str)))
		      break;
		    else
		      ls = g_list_next(ls);		      
		  }
	      }
	    break;
	    
	  case MAIL:
	    if (item->mail_list)
	      {
		ls = item->mail_list;
		
		while (ls)
		  {
		    str = g_strdup(ls->data);
		    
		    if ((find = rubrica_match(str)))
		      break;
		    else
		      ls = g_list_next(ls);		      
		  }
	      }
	    break;
	    
	  case TEL:
	    if (item->tel_list)
	      {
		ls = item->tel_list;
		
		while (ls)
		  {
		    TelNum *tel;
		    
		    tel = ls->data;
		    str = g_strdup(tel->numero_telefono);
		    
		    if ((find = rubrica_match(str)))
		      break;
		    else
		      ls = g_list_next(ls);		      
		  }
	      }
	    break;
	    
	  default:
	    break;
	  }
	
	if (find)
	  {
	  /* 	     aggiungi l'item trovato alla clist
	  */
	    coda = g_list_append(coda, (gpointer)pos);

	    rubrica_search_insert_item(result_clist, item->scheda, str);
	    
	    g_free(str);
	    trovati++;
	  }
	
	find = FALSE;
	
	list = g_list_next(list);
	pos++;
      }
  /* stampo il numero di elementi trovati */
  g_print(_("match element: %d"), trovati);
  buffer = g_strdup_printf(_(" Item(s) found: %d"), trovati);
  gtk_label_set_text(GTK_LABEL(label_result), buffer);	
  g_free(buffer);
}

void rubrica_search_insert_item(GtkWidget *result_clist, gchar *card, gchar *str)
{
  gchar *buffer[3];
  GtkWidget *clist;
  gchar *titolo;

  g_return_if_fail(str != NULL);
  g_return_if_fail(card != NULL);
  g_return_if_fail(result_clist != NULL);

  clist = result_clist;

  titolo = NULL;
  buffer[0] = g_strdup(card);
  buffer[1] = g_strdup(str);
  buffer[2] = NULL;

  switch (where)
    {
    case NAME:
      titolo = g_strdup(_("Name"));
      break;

    case SURNAME:
      titolo =  g_strdup(_("Surname"));
      break;

    case STATE:
      titolo = g_strdup( _("State"));
      break;    

    case CITY:
      titolo = g_strdup(_("City"));
      break;   

    case PROF:
      titolo = g_strdup(_("Profession"));
      break; 
   
    case WEB:
      titolo = g_strdup(_("Url"));
      break;    
      
    case MAIL:
      titolo = g_strdup(_("Email"));
      break;

    case TEL:
      titolo = g_strdup(_("Telephone"));
      break;

    case CARD:
    default:
      break;
    }

  gtk_clist_set_column_title(GTK_CLIST(clist), 1, titolo);
  gtk_clist_append(GTK_CLIST(clist), buffer);  

  /* eliminare g_free se la lista non funziona bene */
  g_free(titolo);
}

void rubrica_search_view_item(gint i)
{
  RubricaItem *r;
  gint pos;
  /*  GtkCList *clist; */

  pos = (gint) g_list_nth_data(coda,i);

  r = rubrica_prendi_elemento(pos);
  /*  clist = rubrica_vista_get_dati(); */ 

  rubrica_mostra_scheda(r);  
}

void rubrica_free_coda(GList *alias)
{     
  g_return_if_fail(alias != NULL);

  if (alias->next)
    rubrica_free_coda(alias->next);
  else
      g_list_free_1(alias);
}

               
